/* FILE: dspasm.h               Copyright (c), Symmetric Research, 1998
 *
 * DSPASM externs, typedefs, and protos.
 *
 *
 * NOTE:
 *
 * The defined constants in this header file are often array indexes in
 * output.c.  Don't make changes without mataining consistency.
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "dsptools.h"


/* OS specific file handling defines */

#ifdef SROS_MSDOS

typedef unsigned SRFILEHANDLE;
#define SrAlloc( nbytes )                    DOS_alloc( (nbytes>>4) + 1 )
#define SrCreate( filename )                 DOS_create( filename )
#define SrOpen( filename )                   DOS_open( filename )
#define SrClose( handle )                    DOS_close( handle )
#define SrRead( handle, buffer, nbytes )     DOS_read(handle,buffer,nbytes)
#define SrWrite( handle, buffer, nbytes )    DOS_write(handle,buffer,nbytes)
#define SrReset( handle )                    DOS_reset( handle )
#define SRFAR far

#else

typedef FILE* SRFILEHANDLE;
#define SUCCESS  0
#define SrAlloc( nbytes )                    malloc( nbytes )
#define SrCreate( filename )                 fopen( filename, "wb" )
#define SrOpen( filename )                   fopen( filename, "rb" )
#define SrClose( handle )                    fclose( handle )
#define SrRead( handle, buffer, nbytes )     fread(buffer,1,nbytes,handle)
#define SrWrite( handle, buffer, nbytes )    fwrite(buffer,1,nbytes,handle)
#define SrReset( handle )                   (fseek(handle,0,SEEK_SET)==SUCCESS)
#define SRFAR 

#endif

   
/****************************** INPUT.C ************************************/

#ifdef SROS_MSDOS

/* DOS memory allocation and file helper functions. */

char SRFAR *DOS_alloc( unsigned nparagraphs);
int         DOS_open(   char *filename);
int         DOS_create( char *filename);
unsigned    DOS_read(  int handle, void SRFAR *buffer, unsigned nbytes);
unsigned    DOS_write( int handle, void SRFAR *buffer, unsigned nbytes);
int         DOS_reset( int handle);
int         DOS_close( int handle);

#endif


/* Input file and buffer functions. */

int

    allocate_input_buffer( void),
          read_input_file( void),
          more_input_file( void),
         reset_input_file( void),
              end_of_file( void);




/* Character classification and token gathering. */

void
           next( void),
          space( void),
        comment( void),

        advance( void);






/* Indirect memory reference data type. */

typedef

    struct mem { int rP, rI, rev; } MEM;





/* Register, constant, and keyword parsing. */

int

         match( int c),
          scan( char *s),

           reg( int *rN),
         pcreg( int *rH),
           acc( int *aN),

           mem( MEM *m),
         memXY( MEM *m),
          memZ( MEM *m);



extern

     unsigned resolved;


int
           hex( unsigned long *N),
          dspf( float *value),
          ieee( float *value);




typedef

    struct mnemonic { char *entry; int value; } MNEMONIC;



int
 
        search( MNEMONIC *table, int *value),
 
               datatype( int *value),
    conditional_keyword( int *value),
    instruction_keyword( int *value),
                control( int *value),
                 ioreg1( int *value),
                 ioreg2( int *value),
                inc_dec( int *value),
                compare( int *value),

           dyadic_arith( int *value),
          dyadic_lshift( int *value),
          dyadic_ashift( int *value),

                  arith( int *value),
                 lshift( int *value),
                 ashift( int *value),

                   cond( int *value),
                 cacond( int *value),
                  relop( int *value),

                   size( int *value),
                   sign( int *value),

                special( int *value);



#define BYTE            0               /* token values */
#define WORD            1
#define LONG            2
#define DSPF            3
#define IEEE            4

#define IF              0
#define WHILE           1
#define FOR             2
#define DO              3
#define LOOP            4

#define NOP             0
#define CALL            1
#define RETURN          2
#define GOTO            3
#define IRETURN         4
#define NOPS_ON         5
#define NOPS_OFF        6
#define EMIT            7
#define IFF             8







/* Error and warning messages. */

void

        error( char *message),
  error_input( char *message),

       syntax( char *message),
      warning( char *message);





/* Hex expression analyser. */

int

    hexval( unsigned long *n),
    hexexp( unsigned long *n),
     value( unsigned long *n),
     hexop( int *value),
   moreexp( unsigned long *value1, int op1),
      eval( unsigned long *value1, int op1, unsigned long value2);





/* Global and local symbol tables. */

#define NHASH                   211             /* hash table size      */

#define CONS_SYM                0x0001          /* global symbol types  */
#define BYTE_SYM                0x0002
#define WORD_SYM                0x0004
#define LONG_SYM                0x0008
#define DSPF_SYM                0x0010
#define IEEE_SYM                0x0020
#define FUNC_SYM                0x0040
#define GOTO_SYM                0x0080

#define REGISTER_SYM            0x0100          /* local symbol types   */
#define STRUCTURE_SYM           0x0200

#define RESOLVED                0x0001          /* symbol flags         */


typedef struct field {

        struct field *next;

            char *name; unsigned long value;

        } FIELD;


typedef struct symbol {

        struct symbol *next;

            char *name; unsigned short type, flags;

                union { unsigned long l; FIELD *p; } value;

        } SYMBOL;



extern SYMBOL

        *gsymbol[NHASH],        /* global hash table */
        *lsymbol[NHASH];        /* local hash table */




unsigned

         token_hash( void);

int
        token_equal( char *s);


SYMBOL
            *salloc( void),

              *find( unsigned type, SYMBOL *table[]),
             *enter( unsigned type, SYMBOL *table[]);

void
              purge( SYMBOL *table[]);


FIELD
            *falloc( void),

        *find_field( FIELD *head),
       *enter_field( FIELD *head);









/****************************** DSPASM.C ***********************************/


/* DOS command line flags. */

extern char

        *filename;

extern unsigned

        pflag,
        nflag,
        hflag,
        tflag,
        lflag,
        sflag,
        saflag,
        scflag;

extern unsigned

        pass,
        nops;




/* Data type sizes in bytes. */

extern

        int datasize[];



/* Parsing functions. */

int
            program( void),
                org( void),

           constant( void),

             global( void),
        initializer( int type, int dflag, unsigned long N),
      initial_value( int type, unsigned long *value),

           function( void),


              local( void),

     named_register( void),
          structure( void),


          statement( void),

        conditional( void),
             ifcond( char SRFAR **fixup),
     check_do_level( void),

        instruction( void),

                cau( void),
          cau_arith( int rD, int rS),

                dau( MEM *Z),
          dau_arith( MEM *Z, int aN, int f, MEM *A);













/****************************** OUTPUT.C ***********************************/


/* Output buffer and file functions. */

extern

    char SRFAR *output;


int
        allocate_output_buffer( void);

void
        write_dso( void),
        write_dsh( void),
        write_dsl( void),

        write_sym( unsigned format);






/* Program location counter and basic emits. */

extern

    unsigned long pc;


void
                 FILE_ID( void),
            CHECK_RECORD( void),
            FINAL_RECORD( void),

                    emit( unsigned long value),
           rewind_output( unsigned long *value),

                    DATA( int type, unsigned long value),

     reset_output_buffer( void);






/* CAU emits. */

#define NONE            0       /* special indexes */
#define ZERO            0
#define DEFAULT         0

#define PC              23
#define PC_SHADOW       24
#define RETURN_REG      14

#define POST_DEC        23
#define POST_INC        24
#define NO_INC          25

#define ONE             23


#define IOC             0       /* control register indexes */
#define DAUC            1

#define IBUF            0       /* ioreg indexes */
#define OBUF            1
#define PDR             2
#define PIOP            3
#define PDR2            4
#define PIR             5
#define PCW             6


#define TO_REG          0       /* memory direction values */
#define FROM_REG        1

#define TO_IO           1
#define FROM_IO         0

#define TO_MEM          1
#define FROM_MEM        0



#define DYADIC          0       /* number of operands */
#define TRIADIC         1

                                /* function indexes */

#define FUNC_00         0       /* 00  +     addition                 */
#define FUNC_01         1       /* 01  *2    mult by 2                */
#define FUNC_02         2       /* 02  -     subtract {N,rS} - rD     */
#define FUNC_03         3       /* 03  #     reverse carry add        */
#define FUNC_04         4       /* 04  -     subtract rD - {N,rS}     */
#define FUNC_05         5       /* 05  -     negation                 */
#define FUNC_06         6       /* 06  &~    and with complement      */
#define FUNC_07         7       /* 07  -     compare w/o store        */
#define FUNC_08         8       /* 08  ^     xor                      */
#define FUNC_09         9       /* 09  >>>1  rotate right thru carry  */
#define FUNC_10         10      /* 10  |     bitwise or               */
#define FUNC_11         11      /* 11  <<<1  rotate left thru carry   */
#define FUNC_12         12      /* 12  >>1   shift right by 1         */
#define FUNC_13         13      /* 13  /2    divide by 2              */
#define FUNC_14         14      /* 14  &     bitwise and              */
#define FUNC_15         15      /* 15  &     bitwise and w/o store    */


#define COND_PL         1       /* condition indexes */
#define COND_MI         2
#define COND_NE         3
#define COND_EQ         4
#define COND_VC         5
#define COND_VS         6
#define COND_CC         7
#define COND_CS         8
#define COND_GE         9
#define COND_LT         10
#define COND_GT         11
#define COND_LE         12
#define COND_HI         13
#define COND_LS         14

#define COND_AUC        15
#define COND_AUS        16
#define COND_AGE        17
#define COND_ALT        18
#define COND_ANE        19
#define COND_AEQ        20
#define COND_AVC        21
#define COND_AVS        22
#define COND_AGT        23
#define COND_ALE        24

#define COND_IBE        31
#define COND_IBF        32
#define COND_OBF        33
#define COND_OBE        34
#define COND_PDE        35
#define COND_PDF        36
#define COND_PIE        37
#define COND_PIF        38
#define COND_SYC        39
#define COND_SYS        40
#define COND_FBC        41
#define COND_FBS        42
#define COND_IREQ1_LO   43
#define COND_IREQ1_HI   44
#define COND_IREG2_LO   45
#define COND_IREG2_HI   46


void

         CAU_0( int cond, int rH, unsigned long N),

         CAU_2( void),

        CAU_3A( int rM, int rH, unsigned long N),

        CAU_3B( unsigned long ninst, unsigned long NREPEAT),
        CAU_3C( unsigned long ninst, int rREPEAT),

         CAU_4( int rH, unsigned long N, int rM),

        CAU_5A( int rD, int rH, unsigned long N),
        CAU_5B( int rD, int rH, unsigned long N),

        CAU_6B( int rD, int operands, int rS, int func, int rT),
        CAU_6D( int rD, int func, unsigned long N),

        CAU_7A( int dir, int size, int rCAU, unsigned long N),
        CAU_7B( int dir, int size, int rCAU, MEM *m),

        CAU_7C( int dir, int size, int rIO, int rCAU),
        CAU_7D( int dir, int size, int rIO, MEM *m),

        CAU_8A( int rH, unsigned long M),
        CAU_8B( int rH, unsigned long M),

        CAU_8C( unsigned long addr, int rM),

    R32_CALL_FIX( int return_reg);





/* CAU custom emits. */

void
            NOPS( int n),
             LOP( void),

          ASHIFT( int rD, int func, int rS, unsigned long N),
          LSHIFT( int rD, int func, int rS, unsigned long N),

         IFF_END( unsigned long cau_instruction, int cacond),

        IF_BEGIN( int cond),
          IF_END( char SRFAR *if_fixup),
      ELSE_BEGIN( char SRFAR *if_fixup),
        ELSE_END( char SRFAR *else_fixup),

       WHILE_END( char SRFAR *top, char SRFAR *fixup),

         FOR_END( char SRFAR *top, char SRFAR *fixup, unsigned long inc),

    DO_REG_BEGIN( int rREPEAT),
    DO_IMM_BEGIN( unsigned long NREPEAT),
          DO_END( char SRFAR *fixup),

        LOOP_END( char SRFAR *top, int rM);








/* DAU emits. */

#define DIRECT          0       /* special indexes */

#define DUPLICATE       15

#define aM_ZERO         4
#define aM_ONE          5
#define aM_FMT4         6

#define DAU_IBUF        4
#define DAU_OBUF        5
#define DAU_PDR         6

                                /* special function indexes */

#define SPECIAL_00      0       /* 00 ic              */
#define SPECIAL_01      1       /* 01 oc              */
#define SPECIAL_02      2       /* 02 float           */
#define SPECIAL_03      3       /* 03 int             */
#define SPECIAL_04      4       /* 04 round           */
#define SPECIAL_05      5       /* 05 ifalt           */
#define SPECIAL_06      6       /* 06 ifaeq           */
#define SPECIAL_07      7       /* 07 ifagt           */
#define SPECIAL_08      8       /* 08 reserved        */
#define SPECIAL_09      9       /* 09 reserved        */
#define SPECIAL_10      10      /* 10 float24         */
#define SPECIAL_11      11      /* 11 int24           */
#define SPECIAL_12      12      /* 12 ieee            */
#define SPECIAL_13      13      /* 13 dsp             */
#define SPECIAL_14      14      /* 14 seed            */
#define SPECIAL_15      15      /* 15 reserved        */


extern

        MEM NO_WRITE, Z_OBUF, Z_PDR, NO_XY;



/* Format 1, mac with accumulator, [Z] = aN = [+,-] Y {+,-} aM*X */

#define DAU_1( Z, aN, f, Y, s, aM, X)   DAU_X( 1, aM, f, s, aN, X, Y, Z)



/* Format 3, mac with memory ops, [Z] = aN = [+,-] aM {+,-} X*Y */

#define DAU_3( Z, aN, f, aM, s, X, Y)   DAU_X( 3, aM, f, s, aN, X, Y, Z)



/* Format 2, tap update with mac, aN = [+,-] aM {+,-} (Z=Y)*X */

#define DAU_2( aN, f, aM, s, Z, Y, X)   DAU_X( 2, aM, f, s, aN, X, Y, Z)



/* Format 4, tap update with add, aN = [+,-] (Z=Y) {+,-} X */

#define DAU_4( aN, f, Z, Y, s, X)       DAU_X( 4, aM_FMT4, f, s, aN, X, Y, Z)





void

   DAU_X( int fmt, int aM, int f, int s, int aN, MEM *X, MEM *Y, MEM *Z),

       DAU_5( MEM *Z, int aN, int func, MEM *Y);

