/* FILE: dspmon.dsp             Copyright (c), Symmetric Research, 1995
 *
 * This is the DSP32C portion of the DSPMON monitor.  It controls
 * breakpoint execution of the chip, rolling out the registers and
 * signaling the PC when a breakpoint has occurred.
 *
 * When reading this code, be aware that it has been very carefully
 * written not to modify the CAU and DAU flags.  Only instructions that
 * do not change the flags have been used.  The DSP32C has no mechanism
 * for saving and restoring the processor flags, so writing code that
 * does not modify them is required.
 *
 *
 * NOTES:
 *
 * * BREAK_BASE SETS THE BASE ADDRESS
 *
 * The constant BREAK_BASE defined below sets the base address for where
 * this code resides in the DSP32C address space.  Currently, BREAK_BASE
 * is set to the internal memory block RAM1 in high memory.  This memory
 * is always present at the same same addresses for all memory mode
 * configurations of the DSP32C, and probably is the best choice for
 * most users.
 *
 *
 * * REGISTERS r21 IS TRASHED
 *
 * The value of r21 is trashed when the CAU registers are rolled out and
 * is not restored.  It does not seem possible to preseve the flags and
 * avoid losing at least one register at the same time.  Note that while
 * r1 is used by the code, its value is restored before returning to the
 * user's code.
 *
 *
 * * ASSEMBLE THIS WITH DSPASM's /sc AND /h OPTIONS
 *
 * This file is assembled with the DSPASM /sc symbol table option to
 * create a C include file for dspmon.c, so it can know the values of
 * the symbols from this file.  The /h DSPASM option is also needed to
 * created a hexified version of this code for including in dspmon.c.
 *
 *
 *
 */


/* The following constant sets the break address for the breakpoint code. */

constant

          BREAK_BASE    = 0xFFF800,     /* breakpoint base address */
                regs    = cau_stack;    /* base of registers for the PC */




/* Goto break_wait instruction for the end of the breakpoint code. */

constant

  GOTO_BREAK_WAIT_HW    = 0xA000 | ((break_wait&0xFF0000)/0x800),
  GOTO_BREAK_WAIT_LW    = (break_wait&0xFFFF);




/* Breakpoint code. */

<BREAK_BASE>

long
        signal = 0, /* breakpoint signal */

            cau_stack[0x16],    /* register and flag stack */

                 dau_stack[0x4],

                    cau_flags_n,
                    cau_flags_z,
                    cau_flags_v,
                    cau_flags_c,

                    dau_flags_n,
                    dau_flags_z,
                    dau_flags_v,
                    dau_flags_u;



break {

   nops_off;


        /* Save the cau registers. */

        r21 = cau_stack;

          *r21++ = r1;
          *r21++ = r2;
          *r21++ = r3;
          *r21++ = r4;
          *r21++ = r5;
          *r21++ = r6;
          *r21++ = r7;
          *r21++ = r8;
          *r21++ = r9;
          *r21++ = r10;
          *r21++ = r11;
          *r21++ = r12;
          *r21++ = r13;
          *r21++ = r14;
          *r21++ = r15;
          *r21++ = r16;
          *r21++ = r17;
          *r21++ = r18;
          *r21++ = r19;
          *r21++ = r20;
          *r21++ = r21;
          *r21++ = r22;     /* = ivtp */


        /* Save the dau registers. */

        r1 = dau_stack;

          *r1++ = a0 = ifalt( a0); nop;
          *r1++ = a1 = ifalt( a1); nop;
          *r1++ = a2 = ifalt( a2); nop;
          *r1++ = a3 = ifalt( a3); nop;


        /* Save the cau flags, nzvc. */

        r1 = cau_flags_n; if ( mi ) { r21 = 0; r21 = 1; } *r1 = r21;
        r1 = cau_flags_z; if ( eq ) { r21 = 0; r21 = 1; } *r1 = r21;
        r1 = cau_flags_v; if ( vs ) { r21 = 0; r21 = 1; } *r1 = r21;
        r1 = cau_flags_c; if ( cs ) { r21 = 0; r21 = 1; } *r1 = r21;


        /* Save the dau flags, nzvu. */

        r1 = dau_flags_n; if ( alt ) { r21 = 0; r21 = 1; } *r1 = r21;
        r1 = dau_flags_z; if ( aeq ) { r21 = 0; r21 = 1; } *r1 = r21;
        r1 = dau_flags_v; if ( avs ) { r21 = 0; r21 = 1; } *r1 = r21;
        r1 = dau_flags_u; if ( aus ) { r21 = 0; r21 = 1; } *r1 = r21;



        /* Poke the wait loop below with goto break_wait; */

        r1 = break_wait;

                r21 = GOTO_BREAK_WAIT_LW; *r1++ = r21.w;
                r21 = GOTO_BREAK_WAIT_HW; *r1++ = r21.w;



        /* Signal the PC the break has occurred. */

        r1 = signal; r21 = 1; *r1 = r21; nop;



        /* Wait to get restarted.
         *
         * NOTE:
         *
         * The break wait and restart instructions are poked by the
         * PC with the appropriate instructions to return to the user's
         * code when restarting.
         *
         *
         */

        break_wait:

                nop;    /* goto break_wait; or goto break_restart; */
                nop;



        /* Restart. */

        break_restart:

                nop;    /* r1 = saved value; */
                nop;    /* goto old_breakpoint; */
                nop;
}
