/* FILE: dspmath.h              Copyright (c), Symmetric Research, 1995-2000
 *
 * This include file has the data type defines and function protos for
 * using the DSPMATH library from C.  It should ALWAYS be included in
 * C programs calling DSPMATH.
 *
 *
 * * DATA TYPE DEFINES:
 *
 * A number of data type defines have been used to make the function
 * protos more readable.  INPUT and OUTPUT are pointers to arrays and
 * indicate what is passed and returned by the functions.
 *
 *
 * * COMPLEX DATA STRUCTURE:
 *
 * The structure for complex data defined below is called complex_single
 * to avoid conflict with the double precision complex data structure
 * usually found in the C runtime include file math.h.
 *
 *
 * * PUSHING FLOATS BY VALUE:
 *
 * The Microsoft C compiler will try to convert all floats to doubles
 * when making function calls unless coerced not to do so by a function
 * prototype.  Including these protos ensures that all floats passed by
 * value are pushed correctly.
 *
 *
 *
 *
 */




/* FUNCTION AND VARIABLE TYPES:
 *
 * These defines control the function and variable types for the SR DSPIO
 * library.
 *
 * Whether a function is marked for static, DLL export, or DLL import use
 * is controlled by the VISIBILITY define.  With the MS VC 6.0 compiler
 * some acceptable values are:
 *
 *      #define VISIBILITY
 *      #define VISIBILITY      __declspec( dllexport)
 *      #define VISIBILITY      __declspec( dllimport) 
 *
 *
 * The calling convention is controlled by the CALLCONV define.  With the
 * MS VC 6.0 compiler some acceptable values are:
 *
 *      #define CALLCONV
 *      #define CALLCONV        __stdcall
 *
 *
 * A null VISIBILITY string indicates standard static usage, while a
 * null CALLCONV string indicates C calling convention.
 *
 * We assume most users will want to use the library as a DLL from a
 * variety of languages, and so have defined dllimport and stdcall as
 * the defaults below.
 *
 * Users needing to override these defaults should give suitable defines
 * from inside their source code or the compiler command line.
 *
 *
 *
 */

#define VOID                    void
#define INT                     int


#ifdef SROS_LINUX
#define _STATIC 1
#undef _DLLEXPORT
#undef _DLLIMPORT
#endif


#ifdef _STATIC
#define VISIBILITY
#define CALLCONV
#endif

#ifdef _DLLEXPORT
#define VISIBILITY              __declspec( dllexport)
#define CALLCONV                __stdcall
#endif

#ifdef _DLLIMPORT
#define VISIBILITY              __declspec( dllimport)
#define CALLCONV                __stdcall
#endif

#ifndef VISIBILITY
#define VISIBILITY              __declspec( dllimport)
#endif

#ifndef CALLCONV
#define CALLCONV                __stdcall
#endif


#define VARTYPE( type)          VISIBILITY type
#define FUNCTYPE( type)         VISIBILITY type CALLCONV



/* Define __huge keyword only for the 16 bit compiler.
 *
 * The 16 bit Microsoft C\C++ version 1.52 compiler sets _MSC_VER = 800, 
 * while the 32 bit Visual C++ version 6.0 compiler sets _MSC_VER = 1200.
 *
 */

#ifdef SROS_LINUX
#define __huge

#else

#if (_MSC_VER <= 800)
#define __huge    _huge
#else
#define __huge
#endif 

#endif // SROS_LINUX





/* ERROR STATUS FLAG:
 *
 * This flag indicates the status of the most recently completed
 * calculation on the DSP32C board.  See dspmath.c and dspmath.dsp for
 * more details about its possible values.
 *
 *
 */

extern unsigned DSPMATH_status;





/* DUMMY ARRAYS:
 *
 * In some of the DSPMATH library functions these arrays can be used to
 * indicate that special actions should be taken on their particular
 * argument.  See dspmath.c for more details of their use.
 *
 *
 */
 
extern unsigned long NO_PIVOT[];

extern float DEFAULT_TAPER[], SAME_FILTER[];







/* DATA TYPE DEFINES:
 *
 *
 *
 *
 */

struct complex_single { float r, i; };

#define WORD            unsigned
#define FLOAT           float
#define COMPLEX         struct complex_single

#define SINT_ARRAY      short int __huge *
#define INT_ARRAY       int __huge *
#define LONG_ARRAY      unsigned long __huge *

#define DIM             unsigned long
#define INPUT           float __huge *
#define OUTPUT          float __huge *

#define COMPLEX_INPUT   struct complex_single __huge *
#define COMPLEX_OUTPUT  struct complex_single __huge *





/* FUNCTION PROTOS:
 *
 *
 *
 *
 */

FUNCTYPE( INT) init_dspmath( void);

FUNCTYPE( VOID) set_waits( unsigned A, unsigned B);
FUNCTYPE( VOID) add( INPUT x, INPUT y, DIM n, OUTPUT z);
FUNCTYPE( VOID) sub( INPUT x, INPUT y, DIM n, OUTPUT z);
FUNCTYPE( VOID) xpcy( INPUT x, FLOAT c, INPUT y, DIM n, OUTPUT z);
FUNCTYPE( VOID) scale( INPUT x, DIM n, FLOAT c, OUTPUT y);
FUNCTYPE( VOID) iscale( INPUT x, DIM n, FLOAT c, SINT_ARRAY y, WORD mode);
FUNCTYPE( VOID) inner( INPUT x, INPUT y, DIM n, OUTPUT result);
FUNCTYPE( VOID) vmult( INPUT x, INPUT y, DIM n, OUTPUT z);
FUNCTYPE( VOID) mmult( INPUT A, INPUT x, DIM n, DIM m, OUTPUT y);
FUNCTYPE( VOID) AtranA( INPUT A, INPUT b, DIM n, DIM m, OUTPUT ATA, OUTPUT ATb);
FUNCTYPE( VOID) gauss( INPUT A, INPUT b, DIM n, OUTPUT LU, OUTPUT x, LONG_ARRAY p);
FUNCTYPE( VOID) tridiag( INPUT u, INPUT d, INPUT l, INPUT b, DIM n, OUTPUT LU_u, OUTPUT LU_d, OUTPUT LU_l, OUTPUT x);
FUNCTYPE( VOID) fourier( COMPLEX_INPUT A, DIM n, COMPLEX_OUTPUT FFTA, int isign);
FUNCTYPE( VOID) rfourier( INPUT x, DIM n, OUTPUT FFTx, int isign);
FUNCTYPE( VOID) spectrum( INPUT x, INPUT taper, DIM n, OUTPUT xspectrum);
FUNCTYPE( VOID) convolve( INPUT x1, INPUT x2, INPUT filter, DIM n, OUTPUT y1, OUTPUT y2);
FUNCTYPE( VOID) fir( INPUT leader, INPUT x, DIM n, INPUT f, DIM nfilter, OUTPUT y);
FUNCTYPE( VOID) poly_array( INPUT c, DIM nc, FLOAT xstart, FLOAT delta, OUTPUT y, DIM n);
FUNCTYPE( VOID) inv_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n);
FUNCTYPE( VOID) sin_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n);
FUNCTYPE( VOID) cos_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n);
FUNCTYPE( VOID) sqrt_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n);
FUNCTYPE( VOID) log10_array( FLOAT xstart, FLOAT delta, OUTPUT y, DIM n);
FUNCTYPE( VOID) vinv( INPUT x, DIM n, OUTPUT y);
FUNCTYPE( VOID) vsin( INPUT x, DIM n, OUTPUT y);
FUNCTYPE( VOID) vcos( INPUT x, DIM n, OUTPUT y);
FUNCTYPE( VOID) vsqrt( INPUT x, DIM n, OUTPUT y);
FUNCTYPE( VOID) vlog10( INPUT x, DIM n, OUTPUT y);
FUNCTYPE( VOID) polar_to_xy( INPUT rtheta, DIM n, OUTPUT xy);
FUNCTYPE( VOID) interpolate( INPUT array1, DIM n1, OUTPUT array2, DIM n2);
FUNCTYPE( VOID) spline ( INPUT x, INPUT y, OUTPUT h, OUTPUT a, OUTPUT b, OUTPUT c, OUTPUT d, OUTPUT xx, OUTPUT yy, DIM n, DIM nn, FLOAT xdiv);
